#ifndef _XMESHLIB_MAP2RECT_TRAITS_H_
#define _XMESHLIB_MAP2RECT_TRAITS_H_

//#pragma warning (disable : 4786)
#include <list>
#include <algorithm>
#include "Core_Mesh/Mesh.h"
#include "Core_Mesh/Iterators.h"
#include "Core_Mesh/Trait.h"

namespace XMeshLib
{
	class Map2Rect_ETrait:public TraitNode
	{
	public:
		Map2Rect_ETrait()
		{
			tId = ID_Map2Rect_ETrait;			
		}
		~Map2Rect_ETrait(){;}

		static void InitTraits(Mesh * cmesh)
		{
			for (MeshEdgeIterator eiter(cmesh); !eiter.end(); ++eiter)
			{
				Edge * e = *eiter;
				Map2Rect_ETrait * te = (Map2Rect_ETrait *)e->GetTrait(ID_Map2Rect_ETrait);
				if (!te)
				{
					te = new Map2Rect_ETrait;
					e->AddTrait(te); 
				}
			}
		}
		static void FreeTraits(Mesh * cmesh)
		{
			for (MeshEdgeIterator eiter(cmesh); !eiter.end(); ++eiter)
			{
				Edge * e = *eiter;
				e->DelTrait(ID_Map2Rect_ETrait);
			}
		}

		double & weight() { return m_weight; };
		double & kuv() { return m_kuv; };		

	private:
		double m_weight;
		double m_kuv;
	};

	class Map2Rect_VTrait : public TraitNode
	{
	public:
		Map2Rect_VTrait()
		{
			tId = ID_Map2Rect_VTrait;
			m_uv[0] = m_uv[1] = 0;
			//m_dual = NULL;
		}
		~Map2Rect_VTrait(){ ;}	

		static void InitTraits(Mesh * cmesh)
		{
			for (MeshVertexIterator viter(cmesh); !viter.end(); ++viter)
			{
				Vertex * v = *viter;
				Map2Rect_VTrait * tv = (Map2Rect_VTrait *)v->GetTrait(ID_Map2Rect_VTrait);
				if (!tv)
				{
					tv = new Map2Rect_VTrait;
					v->AddTrait(tv); 
				}
			}
		}
		static void FreeTraits(Mesh * cmesh)
		{
			for (MeshVertexIterator viter(cmesh); !viter.end(); ++viter)
			{
				Vertex * v = *viter;
				v->DelTrait(ID_Map2Rect_VTrait);
			}
		}

		double * uv() { return m_uv; };
		//Mesh::tVertex & dual() { return m_dual; };
		double & weight(){return m_weight;};
	private:
		double m_uv[2];		
		//Vertex * m_dual;
		double m_weight;
	};

	class Map2Rect_HETrait : public TraitNode
	{
	public:
		Map2Rect_HETrait()
		{
			tId = ID_Map2Rect_HETrait;
			m_angle=m_weight=0;
		}
		~Map2Rect_HETrait(){ ; }

		static void InitTraits(Mesh * cmesh)
		{
			for (MeshHalfEdgeIterator heiter(cmesh); !heiter.end(); ++heiter)
			{
				HalfEdge * he = *heiter;
				Map2Rect_HETrait * the = (Map2Rect_HETrait *)he->GetTrait(ID_Map2Rect_HETrait);
				if (!the)
				{
					the = new Map2Rect_HETrait;
					he->AddTrait(the); 
				}
			}
		}
		static void FreeTraits(Mesh * cmesh)
		{
			for (MeshHalfEdgeIterator heiter(cmesh); !heiter.end(); ++heiter)
			{
				HalfEdge * he = *heiter;
				he->DelTrait(ID_Map2Rect_HETrait);
			}
		}

		double & cornerAngle() 
		{ 
			return m_angle; 
		}
		double & weight() 
		{
			return m_weight;
		}	
	private:
		double m_angle;
		double m_weight;
	};
}

#endif

