#include <cstdlib>
#include <GL/glut.h>
#include <vector>
#include <fstream>
#include "Core_Mesh/Mesh.h"
#include "Core_Mesh/Iterators.h"
#include <iostream>
#include <hash_map>
#include "Renderer.h"
#include "Core_Mesh/Loop.h"
#include <queue>
#include "CG_SPL/cg_spl.h"
#include "Mapping/Map2Rect.h"

using namespace XMeshLib;

Renderer theRenderer;
Mesh * cmesh;
std::vector<Loop> boundaryLoops;
int xNum;
int cNum;
Map2Rect * m2r;

void renderScene(void);
void mouseMove(int x, int y){theRenderer.MouseMove(x,y);}
void mouseClick(int button , int state, int x, int y){theRenderer.MouseClick(button, state, x, y);}
void reshapeScene(GLsizei width, GLsizei height){theRenderer.ReshapeScene(width, height);}
void keyBoard(unsigned char key, int x, int y){theRenderer.KeyBoard(key,x,y);} 

void HW2();
void LoadTexture(const char file[100]);
void HW3();

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
	glutInitWindowPosition(theRenderer.win_x,theRenderer.win_y);
	glutInitWindowSize(theRenderer.win_width, theRenderer.win_height);
	glutCreateWindow("Homework 2 Solution");
	glutDisplayFunc(renderScene);
	glutReshapeFunc(reshapeScene);
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMove);
	glutKeyboardFunc(keyBoard);

	m2r=NULL;
	cmesh = new Mesh();
	cmesh->read(argv[1]);
	theRenderer.SetMesh(cmesh);
	theRenderer.InitGL();

	HW2();
	HW3();
	
	glutMainLoop();
	delete cmesh;
	if (m2r)
		delete m2r;
	return 1;
}

void renderScene(void) 
{
	theRenderer.RenderAll(boundaryLoops);
}

void LoadTexture(const char file[100])
{
	theRenderer.LoadTexture(file);
}

void HW3()
{
	//(1) Validate the input mesh
	if (boundaryLoops.size()!=1 || cNum != 1 || xNum != 1)
	{
		std::cerr << "Error: Please use a topological disk as input." << std::endl;
		return;
	}
	//(2) Find four corner points
	Vertex * corners[4];
	corners[0] = boundaryLoops[0].he(0)->source();
	double curveLen = 0;
	for (int i=0;i<boundaryLoops[0].lenth(); ++i)
	{
		Halfedge * che = boundaryLoops[0].he(i);
		Vertex * sv = che->source();
		Vertex * tv = che->target();
		Point & p1 = sv->point();
		Point & p2 = tv->point();
		double clen = (p1-p2).norm();
		curveLen += clen;
	}
	double checkPoints[3];
	int cornerInd=0;
	double epsilon = 1e-3;
	checkPoints[0] = curveLen * .25 -epsilon;
	checkPoints[1] = curveLen * .5 -epsilon;
	checkPoints[2] = curveLen * .75  -epsilon;
	double checkLen = 0;
	for (int i=0;i<boundaryLoops[0].lenth(); ++i)
	{
		Halfedge * che = boundaryLoops[0].he(i);
		Vertex * sv = che->source();
		Vertex * tv = che->target();
		Point & p1 = sv->point();
		Point & p2 = tv->point();
		double clen = (p1-p2).norm();
		checkLen += clen;
		if ((checkLen-clen < checkPoints[cornerInd]) && (checkLen >= checkPoints[cornerInd]))
		{
			++cornerInd;
			corners[cornerInd]=tv;
		}
	}
	std::cout << "Four Corners: ";
	for (int i=0;i<4;++i)
		std::cout << "Vertex " << corners[i]->id() << " ";
	std::cout << std::endl;
	//(3) Compute the mapping
	m2r = new Map2Rect(cmesh, corners);
	m2r->Parameterize(0,0,1,0,1,1,0,1,"out.m", boundaryLoops[0], curveLen);
	theRenderer.texU = &(m2r->texU);
	theRenderer.texV = &(m2r->texV);

	//(4) Texture Mapping
	LoadTexture("check.bmp");
}



void HW2()
{
	//Euler Number
	xNum = cmesh->numFaces() - cmesh->numEdges() + cmesh->numVertices();
	std::cout << "Euler # = " << xNum << std::endl;

	//Boundary Number
	stdext::hash_map<Halfedge *, bool> bHEVisited;
	Halfedge * boundHE1 = NULL;
	for (MeshEdgeIterator eit(cmesh); !eit.end(); ++eit)
	{
		Edge * e = *eit;
		if (e->boundary())
		{
			boundHE1 = e->he(0);
			bHEVisited[boundHE1]=false;			
		}
	}

	while(boundHE1)
	{
		Loop bLoop;
		bLoop.TraceBoundaryLoop(boundHE1);
		boundaryLoops.push_back(bLoop);
		//Mark them
		for (int i=0;i<bLoop.lenth();++i)
		{
			bHEVisited[bLoop.he(i)]=true;
		}
		boundHE1=NULL;
		for (stdext::hash_map<Halfedge *, bool>::iterator heit=bHEVisited.begin(); 
			heit!=bHEVisited.end(); ++heit)
		{
			Halfedge * cbhe = heit->first;
			if (bHEVisited[cbhe]==false)
			{
				boundHE1 = cbhe;
				break;
			}
		}
	}
	int b = boundaryLoops.size();		
	std::cout << "Boundary # = " << b << std::endl;

	//Connected Component Number
	std::queue<Face *> flist;
	stdext::hash_map<Face *, bool> fVisited;
	Face * startF;
	for (MeshFaceIterator fit(cmesh); !fit.end();++fit)
	{
		Face * f = *fit;
		fVisited[f]=false;
		startF = f;
	}
	cNum = 0;
	flist.push(startF);
	fVisited[startF]=true;
	while(!flist.empty())
	{
		Face * cf = flist.front();
		flist.pop();
		Halfedge * he = cf->he();
		std::vector<Face *> nfs;
		nfs.clear();
		for (int i=0;i<3;++i)
		{
			Halfedge * the = he->he_twin();
			if (the)
				nfs.push_back(the->face());
			he = he->he_next();
		}
		for (int i=0;i<nfs.size();++i)
		{
			Face * nf = nfs[i];
			if (!fVisited[nf])
			{
				fVisited[nf]=true;
				flist.push(nf);
			}
		}
		if (flist.empty())
		{
			cNum++;
			for (stdext::hash_map<Face *, bool>::iterator vfit=fVisited.begin();
				vfit!=fVisited.end();++vfit)
			{
				Face * f = vfit->first;
				if (!fVisited[f])
				{
					flist.push(f);
					fVisited[f]=true;
					break;
				}
			}
		}
	}
	std::cout << "Connected Component # = " << cNum << std::endl;
}
